"""Django AppConfig for frozen_django

.. module:: frozen_django.apps
.. moduleauthor:: Fahri Reza <i@dozy.moe>

"""
from django.apps import AppConfig


class FrozenDjangoConfig(AppConfig):
    """AppConfig for frozen_django

    """
    name = 'frozen_django'
    verbose_name = "Frozen Django"
